﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using System.Web.Security;
using System.Globalization;
using System.Security.Principal;

namespace SurveyHost.Controllers
{
    [HandleError]
    public class HomeController : BaseController
    {
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult About()
        {
            return View();
        }

        public HomeController(IFormsAuthentication formsAuth, MembershipProvider provider)
        {
            FormsAuth = formsAuth ?? new FormsAuthenticationWrapper();
            Provider = provider ?? Membership.Provider;
        }

        public class FormsAuthenticationWrapper : IFormsAuthentication
        {
            public void SetAuthCookie(string userName, bool createPersistentCookie)
            {
                FormsAuthentication.SetAuthCookie(userName, createPersistentCookie);
            }
            public void SignOut()
            {
                FormsAuthentication.SignOut();
            }
        }

        public interface IFormsAuthentication
        {
            void SetAuthCookie(string userName, bool createPersistentCookie);
            void SignOut();
        }

        public HomeController()
            : this(null, null)
        {
        }

        private IFormsAuthentication FormsAuth { get; set; }

        private MembershipProvider Provider { get; set; }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Login(string username, string password, bool rememberMe)
        {
            // Basic parameter validation
            if (String.IsNullOrEmpty(username))
            {
                ModelState.AddModelError("username", "You must specify a username.");
            }
            if (String.IsNullOrEmpty(password))
            {
                ModelState.AddModelError("password", "You must specify a password.");
            }

            if (ViewData.ModelState.IsValid)
            {
                // Attempt to login
                bool loginSuccessful = Provider.ValidateUser(username, password);

                if (loginSuccessful)
                {
                    FormsAuth.SetAuthCookie(username, rememberMe);
                    return RedirectToAction("Index", "Home");
                }
                else
                {
                    ModelState.AddModelError("_FORM", "The username or password provided is incorrect.");
                }
            }

            // If we got this far, something failed, redisplay form
            ViewData["rememberMe"] = rememberMe;
            return View("Index");
        }

        public ActionResult Logout()
        {

            FormsAuth.SignOut();

            return RedirectToAction("Index", "Home");
        }

        private static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            // See http://msdn.microsoft.com/en-us/library/system.web.security.membershipcreatestatus.aspx for
            // a full list of status codes.
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    return "Username already exists. Please enter a different user name.";

                case MembershipCreateStatus.DuplicateEmail:
                    return "A username for that e-mail address already exists. Please enter a different e-mail address.";

                case MembershipCreateStatus.InvalidPassword:
                    return "The password provided is invalid. Please enter a valid password value.";

                case MembershipCreateStatus.InvalidEmail:
                    return "The e-mail address provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidAnswer:
                    return "The password retrieval answer provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidQuestion:
                    return "The password retrieval question provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidUserName:
                    return "The user name provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.ProviderError:
                    return "The authentication provider returned an error. Please verify your entry and try again. If the problem persists, please contact your system administrator.";

                case MembershipCreateStatus.UserRejected:
                    return "The user creation request has been canceled. Please verify your entry and try again. If the problem persists, please contact your system administrator.";

                default:
                    return "An unknown error occurred. Please verify your entry and try again. If the problem persists, please contact your system administrator.";
            }
        }
    }
}

        //public ActionResult Register()
        //{

        //    ViewData["Title"] = "Register";
        //    ViewData["PasswordLength"] = Provider.MinRequiredPasswordLength;

        //    return View();
        //}

        //[AcceptVerbs(HttpVerbs.Post)]
        //public ActionResult Register(string username, string email, string password, string confirmPassword)
        //{

        //    ViewData["Title"] = "Register";
        //    ViewData["PasswordLength"] = Provider.MinRequiredPasswordLength;

        //    // Basic parameter validation
        //    if (String.IsNullOrEmpty(username))
        //    {
        //        ModelState.AddModelError("username", "You must specify a username.");
        //    }
        //    if (String.IsNullOrEmpty(email))
        //    {
        //        ModelState.AddModelError("email", "You must specify an email address.");
        //    }
        //    if (password == null || password.Length < Provider.MinRequiredPasswordLength)
        //    {
        //        ModelState.AddModelError("password",
        //            String.Format(CultureInfo.CurrentCulture,
        //                 "You must specify a password of {0} or more characters.",
        //                 Provider.MinRequiredPasswordLength));
        //    }
        //    if (!String.Equals(password, confirmPassword, StringComparison.Ordinal))
        //    {
        //        ModelState.AddModelError("_FORM", "The new password and confirmation password do not match.");
        //    }

        //    if (ViewData.ModelState.IsValid)
        //    {
        //        // Attempt to register the user
        //        MembershipCreateStatus createStatus;
        //        MembershipUser newUser = Provider.CreateUser(username, password, email, null, null, true, null, out createStatus);

        //        if (newUser != null)
        //        {
        //            FormsAuth.SetAuthCookie(username, false /* createPersistentCookie */);
        //            return RedirectToAction("Index", "Home");
        //        }
        //        else
        //        {
        //            ModelState.AddModelError("_FORM", ErrorCodeToString(createStatus));
        //        }
        //    }

        //    // If we got this far, something failed, redisplay form
        //    return View();
        //}

//[Authorize]
//public ActionResult ChangePassword()
//{

//    ViewData["Title"] = "Change Password";
//    ViewData["PasswordLength"] = Provider.MinRequiredPasswordLength;

//    return View();
//}

//[Authorize]
//[AcceptVerbs(HttpVerbs.Post)]
//public ActionResult ChangePassword(string currentPassword, string newPassword, string confirmPassword)
//{

//    ViewData["Title"] = "Change Password";
//    ViewData["PasswordLength"] = Provider.MinRequiredPasswordLength;

//    // Basic parameter validation
//    if (String.IsNullOrEmpty(currentPassword))
//    {
//        ModelState.AddModelError("currentPassword", "You must specify a current password.");
//    }
//    if (newPassword == null || newPassword.Length < Provider.MinRequiredPasswordLength)
//    {
//        ModelState.AddModelError("newPassword",
//            String.Format(CultureInfo.CurrentCulture,
//                 "You must specify a new password of {0} or more characters.",
//                 Provider.MinRequiredPasswordLength));
//    }
//    if (!String.Equals(newPassword, confirmPassword, StringComparison.Ordinal))
//    {
//        ModelState.AddModelError("_FORM", "The new password and confirmation password do not match.");
//    }

//    if (ModelState.IsValid)
//    {
//        // Attempt to change password
//        MembershipUser currentUser = Provider.GetUser(User.Identity.Name, true /* userIsOnline */);
//        bool changeSuccessful = false;
//        try
//        {
//            changeSuccessful = currentUser.ChangePassword(currentPassword, newPassword);
//        }
//        catch
//        {
//            // An exception is thrown if the new password does not meet the provider's requirements
//        }

//        if (changeSuccessful)
//        {
//            return RedirectToAction("ChangePasswordSuccess");
//        }
//        else
//        {
//            ModelState.AddModelError("_FORM", "The current password is incorrect or the new password is invalid.");
//        }
//    }

//    // If we got this far, something failed, redisplay form
//    return View();
//}

//public ActionResult ChangePasswordSuccess()
//{

//    ViewData["Title"] = "Change Password";

//    return View();
//}

//public ActionResult Login()
//{

//    ViewData["Title"] = "Login";

//    return View();
//}